package org.example

import com.github.h0tk3y.betterParse.lexer.TokenMatch
import com.github.h0tk3y.betterParse.lexer.DefaultTokenizer
import kotlin.test.Test
import kotlin.test.assertEquals

class LexerTest {

    @Test
    fun testSimpleCharacter() {
        val input = "a"
        val tokens = RegexLexer.tokenizer.tokenize(input).toList()
        assertEquals(1, tokens.size)
        assertEquals(RegexLexer.char, tokens[0].type)
        assertEquals("a", tokens[0].text)
    }

    @Test
    fun testCharacterWithPlus() {
        val input = "a+"
        val tokens = RegexLexer.tokenizer.tokenize(input).toList()
        assertEquals(2, tokens.size)
        assertEquals(RegexLexer.char, tokens[0].type)
        assertEquals("a", tokens[0].text)
        assertEquals(RegexLexer.plus, tokens[1].type)
        assertEquals("+", tokens[1].text)
    }

    @Test
    fun testCharacterWithStar() {
        val input = "b*"
        val tokens = RegexLexer.tokenizer.tokenize(input).toList()
        assertEquals(2, tokens.size)
        assertEquals(RegexLexer.char, tokens[0].type)
        assertEquals("b", tokens[0].text)
        assertEquals(RegexLexer.star, tokens[1].type)
        assertEquals("*", tokens[1].text)
    }

    @Test
    fun testCharacterWithQuestion() {
        val input = "c?"
        val tokens = RegexLexer.tokenizer.tokenize(input).toList()
        assertEquals(2, tokens.size)
        assertEquals(RegexLexer.char, tokens[0].type)
        assertEquals("c", tokens[0].text)
        assertEquals(RegexLexer.question, tokens[1].type)
        assertEquals("?", tokens[1].text)
    }

    @Test
    fun testDot() {
        val input = "."
        val tokens = RegexLexer.tokenizer.tokenize(input).toList()
        assertEquals(1, tokens.size)
        assertEquals(RegexLexer.dot, tokens[0].type)
        assertEquals(".", tokens[0].text)
    }

    @Test
    fun testParentheses() {
        val input = "(d)"
        val tokens = RegexLexer.tokenizer.tokenize(input).toList()
        assertEquals(3, tokens.size)
        assertEquals(RegexLexer.openParen, tokens[0].type)
        assertEquals("(", tokens[0].text)
        assertEquals(RegexLexer.char, tokens[1].type)
        assertEquals("d", tokens[1].text)
        assertEquals(RegexLexer.closeParen, tokens[2].type)
        assertEquals(")", tokens[2].text)
    }

    @Test
    fun testAlternation() {
        val input = "e|f"
        val tokens = RegexLexer.tokenizer.tokenize(input).toList()
        assertEquals(3, tokens.size)
        assertEquals(RegexLexer.char, tokens[0].type)
        assertEquals("e", tokens[0].text)
        assertEquals(RegexLexer.alternation, tokens[1].type)
        assertEquals("|", tokens[1].text)
        assertEquals(RegexLexer.char, tokens[2].type)
        assertEquals("f", tokens[2].text)
    }

    @Test
    fun testComplexPattern() {
        val input = "(a|b)*c+"
        val tokens = RegexLexer.tokenizer.tokenize(input).toList()
        assertEquals(8, tokens.size)
        assertEquals(RegexLexer.openParen, tokens[0].type)
        assertEquals(RegexLexer.char, tokens[1].type)
        assertEquals(RegexLexer.alternation, tokens[2].type)
        assertEquals(RegexLexer.char, tokens[3].type)
        assertEquals(RegexLexer.closeParen, tokens[4].type)
        assertEquals(RegexLexer.star, tokens[5].type)
        assertEquals(RegexLexer.char, tokens[6].type)
    }
}