package org.example

import kotlin.test.Test
import kotlin.test.assertEquals
import com.github.h0tk3y.betterParse.grammar.parseToEnd

class ParserTest {

    @Test
    fun testSimpleCharacter() {
        val input = "a"
        val result = RegexParser().parseToEnd(input)
        assertEquals("a",result.toString())
    }
    @Test
    fun testCharacterWithPlus() {
        val input = "a+"
        val parser = RegexParser()
        val result = parser.parseToEnd(input)
        assertEquals("a+", result.toString())
        assert(result.match("a").isSuccess)
        assert(result.match("aa").isSuccess)
        assert(!result.match("b").isSuccess)
    }
    @Test
    fun testCharacterWithStar() {
        val input = "b*"
        val parser = RegexParser()
        val result = parser.parseToEnd(input)
        assertEquals("b*", result.toString())
        assert(result.match("").isSuccess)
        assert(result.match("b").isSuccess)
        assert(result.match("bb").isSuccess)
        assert(result.match("a").isSuccess)
    }
    @Test
    fun testCharacterWithQuestion() {
        val input = "c?"
        val parser = RegexParser()
        val result = parser.parseToEnd(input)
        assertEquals("c?", result.toString())
        assert(result.match("").isSuccess)
        assert(result.match("c").isSuccess)
        assert(result.match("cc").isSuccess)
    }
    @Test
    fun testDot() {
        val input = "."
        val parser = RegexParser()
        val result = parser.parseToEnd(input)
        assertEquals(".", result.toString())
        assert(result.match("a").isSuccess)
        assert(result.match("1").isSuccess)
        assert(!result.match("").isSuccess)
    }

    @Test
    fun testAlternation() {
        val input = "a|b"
        val parser = RegexParser()
        val result = parser.parseToEnd(input)
        assertEquals("(a|b)", result.toString())
        assert(result.match("a").isSuccess)
        assert(result.match("b").isSuccess)
        assert(!result.match("c").isSuccess)
    }
    @Test
    fun testParentheses() {
        val input = "(d)"
        val parser = RegexParser()
        val result = parser.parseToEnd(input)
        assertEquals("d", result.toString())
        assert(result.match("d").isSuccess)
        assert(!result.match("e").isSuccess)
    }

    @Test
    fun testComplexExpression() {
        val input = "a(b|c)*d+"
        val parser = RegexParser()
        val result = parser.parseToEnd(input)
        assertEquals("a(b|c)*d+", result.toString())
        assert(result.match("ad").isSuccess)
        assert(!result.match("ab").isSuccess)
        assert(result.match("acd").isSuccess)
        assert(result.match("abbbd").isSuccess)
        assert(!result.match("a").isSuccess)
        assert(!result.match("b").isSuccess)
    }
    @Test
    fun testAndThen() {
        val input = "ab"
        val parser = RegexParser()
        val result = parser.parseToEnd(input)
        assertEquals("ab", result.toString())
        assert(result.match("ab").isSuccess)
        assert(!result.match("a").isSuccess)
        assert(!result.match("b").isSuccess)
    }
    @Test
    fun testDotAndPlus() {
        val input = ".+a";
        val parser = RegexParser()
        val result = parser.parseToEnd(input)
        assertEquals(".+a", result.toString())
        assert(!result.match("a").isSuccess)
        assert(result.match("ba").isSuccess)
        assert(result.match("bca").isSuccess)
    }
}